'use strict';

var tslib = require('tslib');
var React = require('react');
var axeCore = require('axe-core');
var store = require('@component-controls/store');
var components = require('@component-controls/components');
var blocks = require('@component-controls/blocks');
var themeUi = require('theme-ui');
var octiconsReact = require('@primer/octicons-react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

/* eslint-disable @typescript-eslint/no-empty-function */
const AxeContext = React.createContext({
    results: {
        violations: [],
        passes: [],
        incomplete: [],
    },
});
const AxeSetContext = React.createContext({
    setResults: () => { },
});
const AxeContextProvider = ({ children }) => {
    const [state, setState] = React.useState({
        violations: [],
        passes: [],
        incomplete: [],
    });
    const [mounted, setMounted] = React.useState(true);
    React.useEffect(() => {
        setMounted(true);
        return () => setMounted(false);
    }, []);
    return (React__default["default"].createElement(AxeContext.Provider, { value: {
            results: state,
        } },
        React__default["default"].createElement(AxeSetContext.Provider, { value: {
                setResults: results => {
                    if (mounted) {
                        setState(Object.assign({}, results));
                    }
                },
            } }, children)));
};
const trimNode = (tag) => {
    const trimmedList = tag.split('>');
    const storyNode = trimmedList.findIndex(s => s.includes('[data-testid="story"]'));
    trimmedList.splice(0, storyNode + 1);
    trimmedList.pop();
    return trimmedList.join('>');
};
const trimNodeList = (tagList) => {
    return tagList.map(el => trimNode(el));
};
const SelectionContext = React.createContext({
    selection: [],
    setSelection: () => { },
    isSelected: () => false,
});
const SelectionContextProvider = ({ children }) => {
    const [selection, setSelection] = React.useState([]);
    const value = React.useMemo(() => ({
        selection,
        setSelection: (newSelection) => {
            setSelection(newSelection);
        },
        isSelected: (targets) => tagSelectedList(selection, targets, true).length > 0,
    }), [selection]);
    return (React__default["default"].createElement(SelectionContext.Provider, { value: value }, children));
};
const tagSelectedList = (selection, tags, selected) => {
    const tagsTrimmed = trimNodeList(tags);
    return selection.filter((e) => {
        return tagsTrimmed.includes(trimNode(e)) === selected;
    });
};

/* eslint-disable react/display-name */
const SelectionCheckbox = ({ target }) => {
    const { isSelected, selection, setSelection } = React.useContext(SelectionContext);
    const checked = isSelected(target);
    const toggleSelection = (selector) => {
        const included = tagSelectedList(selection, selector, true);
        if (included.length) {
            setSelection(tagSelectedList(selection, selector, false));
        }
        else {
            setSelection([...selection, ...selector]);
        }
    };
    return (themeUi.jsx(themeUi.Label, null,
        themeUi.jsx(themeUi.Checkbox, { onChange: () => toggleSelection(target), checked: checked })));
};
const NodesTable = ({ nodes, hideErrorColumns, }) => {
    const columns = React.useMemo(() => [
        {
            Header: '',
            accessor: 'impact',
            id: 'node_target',
            width: 20,
            Cell: ({ row: { original: { target }, }, }) => themeUi.jsx(SelectionCheckbox, { target: target }),
        },
        {
            Header: 'summary',
            accessor: 'failureSummary',
        },
        {
            Header: 'html',
            accessor: 'html',
            Cell: ({ value }) => {
                return (themeUi.jsx(themeUi.Flex, { sx: {
                        '& .token-line': {
                            whiteSpace: 'break-spaces',
                        },
                    } },
                    themeUi.jsx(components.SyntaxHighlighter, { language: "markup" }, value)));
            },
        },
        {
            Header: 'target',
            accessor: 'target',
            Cell: ({ value }) => {
                return (themeUi.jsx(themeUi.Flex, { css: {
                        flexWrap: 'wrap',
                    } }, value &&
                    value.map(target => {
                        const selector = trimNode(target);
                        return (themeUi.jsx(components.Tag, { key: `${target}`, color: "lightgrey", variant: "tag.rightmargin", title: selector, sx: {
                                display: 'block',
                                overflow: 'hidden',
                                maxWidth: '400px',
                                whiteSpace: 'nowrap',
                                textOverflow: 'ellipsis',
                            } }, selector));
                    })));
            },
        },
    ], []);
    return (themeUi.jsx(themeUi.Box, { sx: {
            backgroundColor: 'shadow',
        } },
        themeUi.jsx(components.Table, { data: nodes, columns: columns, hiddenColumns: hideErrorColumns ? ['failureSummary', 'html'] : ['html'] })));
};

/* eslint-disable react/display-name */
const impactColors = {
    minor: {
        color: 'status_disabled',
        icon: octiconsReact.InfoIcon,
    },
    moderate: {
        color: 'status_todo',
        icon: octiconsReact.IssueOpenedIcon,
    },
    serious: {
        color: 'status_skipped',
        icon: octiconsReact.AlertIcon,
    },
    critical: {
        color: 'status_failed',
        icon: octiconsReact.StopIcon,
    },
};
const ResultsTable = ({ results, hideErrorColumns, expanded, }) => {
    const renderRowSubComponent = React.useCallback(({ row }) => (themeUi.jsx(NodesTable, { nodes: row.original.nodes, hideErrorColumns: hideErrorColumns })), [hideErrorColumns]);
    const table = React.useMemo(() => {
        const columns = [
            {
                id: 'selector',
                accessor: 'id',
                width: 20,
                Cell: ({ row }) => (themeUi.jsx(themeUi.Flex, Object.assign({}, row.getToggleRowExpandedProps(), { sx: {
                        pl: 2,
                    } }), row.isExpanded ? themeUi.jsx(octiconsReact.ChevronDownIcon, null) : themeUi.jsx(octiconsReact.ChevronRightIcon, null))),
            },
            {
                Header: 'impact',
                accessor: 'impact',
                width: 80,
                Cell: ({ value }) => {
                    const impact = value ? impactColors[value] : undefined;
                    return (impact && (themeUi.jsx(components.Tag, { color: impact.color },
                        themeUi.jsx(impact.icon, null),
                        themeUi.jsx(themeUi.Text, { sx: {
                                pl: impact ? 2 : 0,
                            } }, value))));
                },
            },
            {
                Header: 'id',
                accessor: 'id',
                Cell: ({ value }) => (themeUi.jsx(components.CopyContainer, { value: value },
                    themeUi.jsx(components.Tag, { color: "primary" }, value))),
            },
            {
                Header: 'description',
                accessor: 'description',
                Cell: ({ value, row: { original: { helpUrl }, }, }) => (themeUi.jsx("p", null,
                    value,
                    themeUi.jsx(components.ExternalLink, { sx: { ml: 1 }, href: helpUrl, "aria-label": "view deque description of rule" }, "..."))),
            },
            {
                Header: 'tags',
                accessor: 'tags',
                Cell: ({ value }) => (themeUi.jsx(themeUi.Flex, { css: {
                        flexWrap: 'wrap',
                    } }, value &&
                    value.map(tag => (themeUi.jsx(components.Tag, { key: `${tag}`, color: "gray", variant: "tag.rightmargin" }, tag))))),
            },
        ];
        return (themeUi.jsx(components.Table, { itemsLabel: "a11y tests", data: results || [], expanded: expanded, columns: columns, hiddenColumns: hideErrorColumns ? ['impact', 'tags'] : ['tags'], renderRowSubComponent: renderRowSubComponent }));
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [results === null || results === void 0 ? void 0 : results.length, hideErrorColumns, renderRowSubComponent]);
    return table;
};
const ViolationsTable = () => {
    const { results: { violations }, } = React.useContext(AxeContext);
    const expanded = violations.reduce((acc, _, index) => (Object.assign(Object.assign({}, acc), { [`${index}`]: true })), {});
    // by default expand all to display all errors
    return themeUi.jsx(ResultsTable, { results: violations, expanded: expanded });
};
const PassesTable = () => {
    const { results: { passes }, } = React.useContext(AxeContext);
    return themeUi.jsx(ResultsTable, { results: passes, hideErrorColumns: true });
};
const IncompleteTable = () => {
    const { results: { incomplete }, } = React.useContext(AxeContext);
    return themeUi.jsx(ResultsTable, { results: incomplete });
};

const HighlightNode = (_a) => {
    var { selection } = _a, rest = tslib.__rest(_a, ["selection"]);
    const styles = React.useMemo(() => {
        const selectors = selection.map(c => {
            const style = c.split('.story-render-container');
            let selector;
            if (style.length >= 2) {
                selector = `.story-render-container${style[style.length - 1] ? style[style.length - 1] : ':first-child'}`;
            }
            else if (c === ':root') {
                selector = '.story-render-container > * ';
            }
            else {
                selector = c;
            }
            return `${selector}`;
        });
        return selectors.length
            ? {
                [`${selectors.join(', ')}`]: {
                    outline: '3px dotted red',
                    outlineOffset: '3px',
                },
            }
            : undefined;
    }, [selection]);
    return themeUi.jsx(themeUi.Box, Object.assign({ sx: styles }, rest));
};
const HighlightSelector = ({ children }) => {
    const { selection } = React.useContext(SelectionContext);
    return themeUi.jsx(HighlightNode, { selection: selection }, children);
};

/* eslint-disable react/display-name */
const PanelState = {
    violations: ViolationsTable,
    passes: PassesTable,
    incomplete: IncompleteTable,
};
/**
 * Displays the [axe](https://github.com/dequelabs/axe-core) ally test results
 */
const BaseAllyBlock = ({ children }) => {
    const { results: { violations, passes, incomplete }, } = React.useContext(AxeContext);
    const defaultState = React.useCallback(() => violations.length
        ? 'violations'
        : incomplete.length
            ? 'incomplete'
            : 'passes', [incomplete.length, violations.length]);
    const [state, setState] = React.useState(defaultState());
    React.useEffect(() => {
        setState(defaultState());
    }, [defaultState]);
    const numPanels = (violations.length ? 1 : 0) +
        (incomplete.length ? 1 : 0) +
        (passes.length ? 1 : 0);
    const nextState = () => {
        switch (state) {
            case 'violations':
                if (passes.length) {
                    return 'passes';
                }
                return 'incomplete';
            case 'passes':
                if (incomplete.length) {
                    return 'incomplete';
                }
                return 'violations';
            case 'incomplete':
                if (violations.length) {
                    return 'violations';
                }
                return 'passes';
        }
    };
    const Panel = PanelState[state];
    return (React__default["default"].createElement(components.ActionContainer, { actions: numPanels > 1
            ? [
                {
                    node: `view ${nextState()}`,
                    id: 'panel',
                    'aria-label': `display the accessibility ${nextState()}`,
                    onClick: () => setState(nextState()),
                },
            ]
            : undefined },
        React__default["default"].createElement(HighlightSelector, null, children),
        numPanels > 0 && React__default["default"].createElement(Panel, null)));
};

const RenderStory = ({ axeOptions, storyId, }) => {
    const storyRef = React.useRef(null);
    const { setResults } = React.useContext(AxeSetContext);
    const isRunning = React.useRef(false);
    const collectResults = () => {
        var _a;
        const canvas = (_a = storyRef.current) === null || _a === void 0 ? void 0 : _a.firstChild;
        if (canvas && isRunning.current === false) {
            isRunning.current = true;
            axeCore.configure(axeOptions || {});
            components.resetTabCounter();
            axeCore.run(canvas)
                .then(results => {
                const { passes, violations, incomplete } = results;
                setResults({ passes, violations, incomplete });
                setTimeout(() => (isRunning.current = false), 1000);
            })
                .catch(e => {
                console.error('error running axe-core', e);
                isRunning.current = false;
            });
        }
    };
    const onRender = () => {
        try {
            collectResults();
        }
        catch (e) { }
    };
    return (React__default["default"].createElement(blocks.Story, { key: storyId, id: storyId, ref: storyRef, onRender: onRender }));
};
/**
 * Story block container that displays displays the [axe](https://github.com/dequelabs/axe-core) ally test results
 */
const AllyBlock = (_a) => {
    var { axeOptions, id, name } = _a, props = tslib.__rest(_a, ["axeOptions", "id", "name"]);
    const story = store.useStory({ id, name });
    React.useEffect(() => {
        return () => {
            axeCore.cleanup();
        };
    }, []);
    return story ? (React__default["default"].createElement(blocks.StoryBlockContainer, Object.assign({ story: story }, props),
        React__default["default"].createElement(AxeContextProvider, null,
            React__default["default"].createElement(SelectionContextProvider, null,
                React__default["default"].createElement(BaseAllyBlock, { options: axeOptions },
                    React__default["default"].createElement(RenderStory, { storyId: story.id, axeOptions: axeOptions })))))) : null;
};

exports.AllyBlock = AllyBlock;
